/** 
 * @file
 *
 * @brief Test basic functionalities of IWING-MRF board
 *
 * This firmware interacts with the user via UART.  Functionalities to be
 * tested include:
 *    - UART
 *    - LEDs
 *    - Sensor inputs
 *    - Radio TX/RX
 */
#include <stdio.h>
#include <string.h>

#include <motelib/system.h>
#include <motelib/button.h>
#include <motelib/uart.h>
#include <motelib/sensor.h>
#include <motelib/actor.h>
#include <motelib/led.h>
#include <motelib/radio.h>
#include <motelib/timer.h>
#include "pt/pt.h"

#if defined(PLATFORM_IWING_JN)
#define SENSOR_DIGITAL_START SENSOR_D0
#define SENSOR_DIGITAL_COUNT 18
#define SENSOR_ANALOG_START  SENSOR_A1
#define SENSOR_ANALOG_COUNT  6
#define ACTOR_START  ACTOR_0
#define ACTOR_COUNT  18
#elif defined(PLATFORM_IWING_MRF)
#define SENSOR_DIGITAL_START SENSOR_0
#define SENSOR_DIGITAL_COUNT 6
#define SENSOR_ANALOG_START  SENSOR_0
#define SENSOR_ANALOG_COUNT  6
#define ACTOR_START  ACTOR_0
#define ACTOR_COUNT  6
#else
#define SENSOR_DIGITAL_START 0
#define SENSOR_DIGITAL_COUNT 6
#define SENSOR_ANALOG_START  0
#define SENSOR_ANALOG_COUNT  6
#define ACTOR_START  0
#define ACTOR_COUNT  3
#endif

struct pt sensor_pt;
uint8_t flag = 0;
Timer t;
#define LEFT_TYPE 2
#define RIGHT_TYPE 3
#define RELOAD_TYPE 4
#define SHOOT_TYPE 5
#define REQ 11
#define RES 12
//////////////////////////////////////////////////
#ifdef PLATFORM_IWING_MRF
#include <avr/pgmspace.h>
#define printf(fmt,args...) printf_P(PSTR(fmt),##args)
#endif

//////////////////////////////////////////////////
#define READ_ANALOG(sensor, stored_var) \
    sensorRequestAnalog(sensor, NULL); \
    PT_WAIT_WHILE(pt, sensorAnalogWaiting(sensor)); \
    stored_var = sensorAnalogResult(sensor);

//////////////////////////////////////////////////
#define WAIT_INPUT(stored_var) \
    PT_WAIT_UNTIL(pt, uartInputLen() > 0); \
    stored_var = uartReadByte();

char *MSG = "Hello";
char *actorKeys = "0123456789abcdefghijklmn";


//////////////////////////////////////////////////
void buttonHandler(ButtonStatus status)
{
    if (status == BUTTON_PRESSED)
        printf("USER BUTTON is pressed.\r\n");
    else
        printf("USER BUTTON is released.\r\n");
}

void Timerfired(Timer x){
    printf("\n\nSession Expired\n\n");
    flag = 0;
}
//////////////////////////////////////////////////
void recv(Address source, MessageType type, void *message, uint8_t len)
{
    char pass = ((char *)message)[0];
    //printf("%c\n",pass);
    if(type == RES && pass == 'a'){
        flag = 1;

        printf("\n\nComfirmation Accept!!\n\n");
        printf(
                "\r\n"
                "Sticky Missile Tower Controller System -- (SMTCS)\r\n"
                "- Press 'a' to turn left\r\n"
                "- Press 'd' to turn right\r\n"
                "- Press 'r' to reload\r\n"
                "- Press 's' to shoot\r\n"
                "READY > "
              );
    timerStart(&t,TIMER_ONESHOT,10000,Timerfired);
    }
}

//////////////////////////////////////////////////
PT_THREAD(sensorTask(struct pt *pt))
{
    static uint8_t i, msgType;
    static uint8_t MSG_LEN;
    char key;
    uint16_t val;

    PT_BEGIN(pt);

    MSG_LEN = strlen(MSG);

    while (1)
    {
        if(flag == 0){
            printf("\nPress any key to send request...\n");
        }
        WAIT_INPUT(key);
        printf("%c\r\n", key);
        if(flag == 1){

                printf(
                    "\r\n"
                    "Sticky Missile Tower Controller System -- (SMTCS)\r\n"
                    "- Press 'a' to turn left\r\n"
                    "- Press 'd' to turn right\r\n"
                    "- Press 'r' to reload\r\n"
                    "- Press 's' to shoot\r\n"
                    "READY > "
                );
                if(key == 'a'){
                    radioRequestTx(BROADCAST_ADDR, LEFT_TYPE, "left", 6, NULL);
                }
                if(key == 'd'){
                    radioRequestTx(BROADCAST_ADDR, RIGHT_TYPE, "right", 6, NULL);
                }

                if(key == 'r'){
                    radioRequestTx(BROADCAST_ADDR, RELOAD_TYPE, "reload", 6, NULL);
                }
                if(key == 's'){
                    radioRequestTx(BROADCAST_ADDR, SHOOT_TYPE, "shoot", 6, NULL);
                }
                if(key =='a' || key =='d' || key == 'r' || key == 's'){
                    timerStop(&t);
                    timerStart(&t,TIMER_ONESHOT,10000,Timerfired);
                }
        }else{
            printf("\nNeed Comfirmation.\n");
            printf("Sending Request..\n");
            radioRequestTx(BROADCAST_ADDR, REQ , "b", 1 ,NULL);
            printf("Request Sent.\n");
        }

    }

    PT_END(pt);
}
//////////////////////////////////////////////////
void scheduleTasks()
{
    sensorTask(&sensor_pt);
}

//////////////////////////////////////////////////
void boot()
{
    timerCreate(&t);
    uartEnable(true, true);  // tx - enabled, rx - enabled
    radioSetRxHandler(recv);
    setLoopRoutine(scheduleTasks);
}
